import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.XMLReaderFactory;
import java.io.*;
import java.util.LinkedList;

public class MiSax4 extends DefaultHandler {

	FileWriter Fichero;
	int profundidad = 0;
	// Nombre del fichero a creae
	String nFich;
	// Autor de la obra que se esta leyendo en el fichero
	StringBuffer autor;
	// Lista de listaAutores de la obra que se esta leyendo en el fichero
	LinkedList<String> listaAutores;
	// Booleano para saber si lo leido es la tag de "Author"
	boolean esAutorOEditor = false;
	// Booleano para saber si lo leido es la tag de "Title"
	boolean esTitulo = false;

	public MiSax4() {
		super();
	}

	public static void main(String args[]) throws Exception {
		XMLReader xr = XMLReaderFactory.createXMLReader();
		MiSax4 handler = new MiSax4();
		xr.setContentHandler(handler);
		// Esto analiza todo lo que le pasemos por terminal
		for (int i = 0; i < args.length; i++) {
			FileReader r = new FileReader(args[i]);
			xr.parse(new InputSource(r));	
		}
	}

	//Acciones a realizar al principio del documento
	public void startDocument() {
	}

	//Acciones a realizar al final del documento
	public void endDocument() {
	}

	//Acciones a realizar al principio de elemento.
	public void startElement(String uri, String name, String qName, Attributes atts) {
		// Incrementamos la profundidad
		profundidad++;
		if(profundidad ==2){
			// Si es profundidad 2, estamos ante un nuevo documento (libro, articulo...)
			// y por tanto inicializamos el LinkedList de nuevo 
			// (borramos los datos que pudiera contener de la lectura anterior)
			listaAutores = new LinkedList<String>();
		// Si no es profundidad 2, comprobamos si estamos leyendo la tag "author" o "editor"
		}else if (qName.equals("author") || qName.equals("editor")) {
			// Si coincide con alguna de esas dos etiquetas, ponemos el atributo a true
			esAutorOEditor = true;
			// E inicializamos el buffer de lectura
			autor = new StringBuffer("");
		// Sino comprobamos si estamos leyendo la tag "title"
		} else if(qName.equals("title")) {
			// Si coincide la etiqueta, ponemos el booleano a true
			esTitulo = true;
			// E intentamos crear el fichero
			try {
				// El nombre del fichero sera el nombre del ultimo autor sin espacios en blanco
				// Si el documento no tiene autores o editores
				if (listaAutores.size()==0){
					// El nombre de fichero sera "Anonimo"
					nFich="Anonimo";
				}else{
					// Si hay autores, guardamos en un string el ultimo autor de la lista de listaAutores del documento
					// que esta siendo analizado
					String ultimoAutor = listaAutores.get(listaAutores.size()-1);
					// Y eliminamos los espacios en blanco
					String ultimoAutorSinEspacios = ultimoAutor.replaceAll("\\s+","");
					// Damos nombre al fichero con las caracteristicas pedidas
					nFich = ultimoAutorSinEspacios;
				}
				File file = new File(nFich+".txt");
				// Si el fichero ya existe, es decir, si el ultimo autor coincide con otros documentos
				if(file.exists()) {
					// Creamos ficheros de la forma NOMBRE(INDICE), siendo INDICE un numero que incrementara
					// en el caso de que exista mas de un fichero con el mismo nombre
					int i = 1;
					do {
						// Fichero que queremos crear
						file = new File(nFich+"("+i+").txt");
						// Si dicho fichero no existe
						if(!file.exists()) {
							// Lo guardamos en la variable y salimos del bucle
							nFich = nFich+"("+i+")";
						} else {
							// Sino seguimos iterando
							i++;
						} // fin if
					} while(file.exists()); // fin do-while
				}// fin if
				Fichero = new FileWriter(nFich+".txt");
				// Escribo en el fichero las comillas de inicio del titulo
				Fichero.write("\"");
			} catch(Exception e) {
			} // fin try
		}// fin if
	}// fin metodo startElement

	//Acciones a realizar al final de un elemento
	public void endElement(String uri, String name, String qName) {
		// Decrementamos la profundidad
		profundidad--;
		// Si llegamos a profundidad 1
		if(profundidad==1) {
			// Intentamos escribir en el fichero los autores
			try {
				int i=0;
				// Guardamos el numero total de autores del documento
				int numAutores = listaAutores.size();
				// Mientras no hayamos llegado al final de la lista seguimos escribiendo 
				// en el fichero
				while (i<numAutores){
					// Ultimo nombre a escribir, no va seguido de coma
					if(i==numAutores-1) {
						Fichero.write(listaAutores.get(i));
					// Resto de nombres a escribir, seguidos de coma
					} else {
						Fichero.write(listaAutores.get(i)+", ");
					}
					// Incrementamos 
					i++;
				}// fin while
				// Cerramos el fichero
				Fichero.close();
			} catch(Exception e) {
			}// fin try
		// Si es una etiqueta "author" o "editor"
		} else if(qName.equals("author") || qName.equals("editor")) {
			esAutorOEditor = false;
			// Guardo el autor o editor en la lista
			listaAutores.add(autor.toString());
		// Si es una etiqueta "title"
		} else if(qName.equals("title")) {
			esTitulo = false;
			// Intento escribir en el fichero las comillas de cierre
			try {
				Fichero.write("\"\n");
			} catch(Exception e) {
			}// fin try
		}// fin if
	}// fin metodo endElement

	//Acciones a realizar cada vez que encuentre caracteres
	public void characters(char ch[], int start, int length) {
		// Si es un autor o editor
		if(esAutorOEditor){
			// Guardamos caracter a caracter el autor
			for(int i=start;i<start+length;i++) {
				autor.append(ch[i]);
			}// fin for
		}// fin if
		// Si es el titulo de la obra o documento
		if(esTitulo) {
			try {
				// Escribimos caracter a caracter el titulo
				for(int i=start;i<start+length;i++) {
					Fichero.write(ch[i]);
				}// fin for
			} catch(Exception e) {
			}// fin try
		} // fin if
	}// fin metodo characters
} // fin clase MiSax4
